@@ -22,6 +22,7 @@ import json
 import csv
 import logging
 import importlib
+import tempfile
 
 from tqdm.auto import tqdm
 from pydantic import BaseModel, Field
@@ -58,6 +59,15 @@ class Turn(BaseModel):
     speaker: Optional[str] = None
     text: str
 
+    def __len__(self):
+        """
+        Returns the number of words in the turn's text.
+
+        :return: Number of words in the text.
+        :rtype: int
+        """
+        return len(self.text.split())
+
     def __str__(self):
         return f"{self.speaker}: {self.text}"
 
@@ -148,9 +158,77 @@ class Dialog(BaseModel):
     turns: Optional[List[Turn]] = Field(default_factory=list)
     events: Optional[List[Event]] = None
     notes: Optional[str] = None
-
     _path: Optional[str] = None
 
+    def to_audio(
+        self,
+        **kwargs: dict
+    ):
+        """
+        Convert the dialogue to an audio dialogue.
+        This is a convenience wrapper around the full `sdialog.audio.pipeline.to_audio` function.
+        All keyword arguments are passed to it.
+        :param dir_audio: Directory path for storing audio outputs.
+        :type dir_audio: str
+        :param dialog_dir_name: Custom name for the dialogue directory.
+        :type dialog_dir_name: str
+        :param dscaper_data_path: Path to dSCAPER data directory.
+        :type dscaper_data_path: Optional[str]
+        :param room_name: Custom name for the room configuration.
+        :type room_name: Optional[str]
+        :param perform_tts: Convert the dialog into audio using the text-to-speech engine.
+        :type perform_tts: Optional[bool]
+        :param perform_room_acoustics: Enable room acoustics simulation and dSCAPER timeline generation.
+        :type perform_room_acoustics: Optional[bool]
+        :param tts_engine: Text-to-speech engine for audio generation.
+        :type tts_engine: BaseTTS
+        :param voice_database: Voice database for speaker selection.
+        :type voice_database: BaseVoiceDatabase
+        :param dscaper_datasets: List of Hugging Face datasets for dSCAPER.
+        :type dscaper_datasets: List[str]
+        :param room: Room configuration for acoustics simulation.
+        :type room: Room
+        :param speaker_positions: Speaker positioning configuration.
+        :type speaker_positions: dict[Role, dict]
+        :param background_effect: Background audio effect type.
+        :type background_effect: str
+        :param foreground_effect: Foreground audio effect type.
+        :type foreground_effect: str
+        :param foreground_effect_position: Position for foreground effects.
+        :type foreground_effect_position: RoomPosition
+        :param kwargs_pyroom: PyRoomAcoustics configuration parameters.
+        :type kwargs_pyroom: dict
+        :param source_volumes: Volume levels for different audio sources.
+        :type source_volumes: dict[SourceType, SourceVolume]
+        :param audio_file_format: Audio file format (wav, mp3, flac).
+        :type audio_file_format: str
+        :param seed: Seed for random number generator.
+        :type seed: int
+        :param re_sampling_rate: Re-sampling rate for the output audio.
+        :type re_sampling_rate: Optional[int]
+        :param recording_devices: The identifiers of the recording devices to simulate.
+        :type recording_devices: Optional[List[Union[RecordingDevice, str]]]
+        :param impulse_response_database: The database for impulse responses.
+        :type impulse_response_database: Optional[ImpulseResponseDatabase]
+        :param override_tts_audio: Override the TTS audio if it already exists.
+        :type override_tts_audio: Optional[bool]
+        :param verbose: Verbose mode for logging.
+        :type verbose: Optional[bool]
+        :return: Audio dialogue with processed audio data.
+        :rtype: "sdialog.audio.dialog.AudioDialog"
+        :raises Exception: If the audio module is not installed.
+        """
+
+        try:
+            from sdialog.audio.pipeline import to_audio
+        except Exception:
+            raise Exception("The audio module is not installed. Please install it with `pip install sdialog[audio]`")
+
+        return to_audio(
+            self,
+            **kwargs
+        )
+
     def __len__(self):
         """
         Returns the number of turns in the dialogue.
@@ -451,6 +529,92 @@ class Dialog(BaseModel):
             else:
                 writer.write(self.description())
 
+    @staticmethod
+    def from_huggingface(repo_id: str,
+                         local_dir: str = None,
+                         collapse_consecutive_speakers: bool = False,
+                         collapse_separator: str = "\n") -> Union[List["Dialog"], Dict[str, List["Dialog"]]]:
+        """
+        Loads dialogues from a HuggingFace dataset.
+
+        This method downloads a dataset from HuggingFace Hub and loads dialogues from it.
+        The dataset must follow the SDialog format with a 'data' folder containing dialogue files.
+        If the data folder contains train/test/val split subdirectories, dialogues are loaded
+        from all splits and returned as a dictionary mapping split names to dialogue lists.
+        Otherwise, all dialogues from the data folder are returned as a single list.
+
+        :param repo_id: HuggingFace repository ID (e.g., "sdialog/Primock-57").
+        :type repo_id: str
+        :param local_dir: Local directory to download to. If None, uses a temporary directory.
+        :type local_dir: str, optional
+        :param collapse_consecutive_speakers: If True, collapses consecutive turns by the same speaker.
+        :type collapse_consecutive_speakers: bool
+        :param collapse_separator: Separator used when collapsing consecutive turns.
+        :type collapse_separator: str
+        :return: List of dialogs or dict mapping splits to lists of dialogs.
+        :rtype: Union[List[Dialog], Dict[str, List[Dialog]]]
+        :raises ImportError: If huggingface_hub is not installed.
+        :raises ValueError: If the dataset is not a valid sdialog dataset.
+        """
+        from huggingface_hub import snapshot_download
+
+        # Use temporary directory if local_dir is not provided
+        temp_dir = None
+        if local_dir is None:
+            temp_dir = tempfile.mkdtemp()
+            local_dir = temp_dir
+
+        try:
+            # Download the dataset
+            snapshot_download(
+                repo_id=repo_id,
+                repo_type="dataset",
+                local_dir=local_dir,
+            )
+
+            # Check if dataset has 'data' folder
+            data_dir = os.path.join(local_dir, "data")
+            if not os.path.exists(data_dir):
+                raise ValueError(f"'{repo_id}' is not a valid sdialog dataset. "
+                                 "Expected 'data' folder not found in downloaded dataset.")
+
+            # Check for split folders (train, test, val)
+            split_folders = ["train", "test", "val"]
+            existing_splits = {}
+
+            for split in split_folders:
+                split_path = os.path.join(data_dir, split)
+                if os.path.exists(split_path) and os.path.isdir(split_path):
+                    existing_splits[split] = split_path
+
+            # If split folders exist, load dialogs from all splits
+            if existing_splits:
+                result = {}
+                for split_name, split_path in existing_splits.items():
+                    dialogs = Dialog.from_file(
+                        split_path,
+                        type="json",
+                        collapse_consecutive_speakers=collapse_consecutive_speakers,
+                        collapse_separator=collapse_separator
+                    )
+                    result[split_name] = dialogs
+                return result
+            else:
+                # No split folders, load all dialogs from data directory
+                dialogs = Dialog.from_file(
+                    data_dir,
+                    type="json",
+                    collapse_consecutive_speakers=collapse_consecutive_speakers,
+                    collapse_separator=collapse_separator
+                )
+                return dialogs
+
+        finally:
+            # Clean up temporary directory if we created one
+            if temp_dir is not None:
+                import shutil
+                shutil.rmtree(temp_dir, ignore_errors=True)
+
     @staticmethod
     def from_file(path: str,
                   type: str = "auto",
@@ -558,7 +722,7 @@ class Dialog(BaseModel):
                     if speaker is None:
                         raise ValueError(f"Missing speaker in row {ix}: {row}")
                     if not text:
-                        logger.warning(f"File '{path}': Empty text in row {ix}: {row}. Skipping this turn.")
+                        logger.debug(f"File '{path}': Empty text in row {ix}: {row}. Skipping this turn.")
                         continue
                     turns.append((speaker.strip(), text.strip()))
 
@@ -723,7 +887,6 @@ class Dialog(BaseModel):
         if speaker.lower() not in self.get_speakers(keep_case=False):
             logger.error(f"The provided speaker '{speaker}' does not exist in the dialogue. "
                          f"Valid speakers are: {self.get_speakers()}")
-            return None
 
         filtered_dialog = self.clone()
         filtered_dialog.turns = [turn for turn in self.turns if turn.speaker.lower() == speaker.lower()]
