@@ -18,15 +18,15 @@ from collections import defaultdict
 from typing import List, Union, Optional, Tuple
 
 from langchain_core.tools import tool
-from langchain_core.messages.base import messages_to_dict
 from langchain_core.language_models.base import BaseLanguageModel
+from langchain_core.messages.base import BaseMessage, messages_to_dict
 from langchain_core.messages import HumanMessage, SystemMessage, AIMessage
 
 from .config import config
 from jinja2 import Template
 
 from . import Dialog, Turn, Event, Instruction, Context
-from .personas import BasePersona, Persona
+from .personas import BasePersona
 from .orchestrators import BaseOrchestrator
 from .interpretability import ResponseHook, ActivationHook, Inspector
 from .util import get_llm_model, is_amazon_model_name, is_huggingface_model_name, set_generator_seed, get_universal_id
@@ -104,7 +104,8 @@ class Agent:
     :param postprocess_fn: Optional function to postprocess each output utterance after calling the LLM
                             (input string, output string).
     :type postprocess_fn: Optional[callable]
-    :param system_prompt: Custom system prompt (optional, otherwise loaded from config).
+    :param system_prompt: Custom system prompt to use as-is (takes precedence over persona;
+                          if provided, persona is disabled and this prompt is used directly).
     :type system_prompt: Optional[str]
     :param model: The LLM or model name to use (defaults to config["llm"]["model"]).
     :type model: Union[str, BaseLanguageModel], optional
@@ -115,7 +116,7 @@ class Agent:
     _STOP_WORD_TEXT = "(bye bye!)"
 
     def __init__(self,
-                 persona: BasePersona = Persona(),
+                 persona: BasePersona = None,
                  name: Optional[str] = None,
                  context: Optional[Union[str, Context]] = None,
                  first_utterance: Optional[Union[str, List[str]]] = None,
@@ -146,9 +147,16 @@ class Agent:
         if preprocessing_fn is not None and not callable(preprocessing_fn):
             raise ValueError("preprocessing_fn must be a callable function that takes a string and outputs a string.")
 
-        if not system_prompt:
+        # Handle system_prompt parameter - if provided, it takes precedence over persona
+        if system_prompt:
+            if persona:
+                logger.warning("Both system_prompt and persona provided. system_prompt takes precedence; "
+                               "persona will be ignored.")
+            persona = None  # Disable persona logic when custom system_prompt is provided
+            system_prompt_template = None
+        else:
             with open(config["prompts"]["persona_agent"], encoding="utf-8") as f:
-                system_prompt_template = Template(f.read())
+                system_prompt_template = Template(f.read() if persona else "")
 
         # Private attributes
         self._system_prompt_template = system_prompt_template
@@ -168,6 +176,7 @@ class Agent:
         self._preprocessing_fn = preprocessing_fn
         self._hook_response_data = None
         self._hook_response_act = defaultdict(lambda: defaultdict(list))
+        self._hook_response_logit = defaultdict(list)
 
         # Public attributes
         self.llm, llm_kwargs = get_llm_model(model_name=model,
@@ -178,24 +187,49 @@ class Agent:
         self.model_info = {"name": str(model), **llm_kwargs}
         self.name = name if name is not None else getattr(persona, "name", None)
         self.persona = persona
-        self.memory = [SystemMessage(self._system_prompt_template.render(
-            persona=self.persona.prompt(),
-            context=self._context,
-            example_dialogs=self._example_dialogs,
-            dialogue_details=self._dialogue_details,
-            response_details=self._response_details,
-            can_finish=self._can_finish,
-            stop_word=self._STOP_WORD
-        ))]
 
-        logger.debug(f"Initialized agent '{self.name}' with model '{str(model)}' "
-                     f"using prompt from '{config['prompts']['persona_agent']}'.")
-        logger.debug("Prompt: " + self.prompt())
+        # Initialize memory based on whether we have a custom system_prompt or persona
+        if system_prompt:
+            self._memory = [SystemMessage(system_prompt)]
+            self._system_prompt = self._memory[0].content
+        elif persona and self._system_prompt_template:
+            self._memory = [SystemMessage(self._system_prompt_template.render(
+                persona=self.persona.prompt(),
+                context=self._context,
+                example_dialogs=self._example_dialogs,
+                dialogue_details=self._dialogue_details,
+                response_details=self._response_details,
+                can_finish=self._can_finish,
+                stop_word=self._STOP_WORD
+            ))]
+            self._system_prompt = self._memory[0].content
+        else:
+            self._memory = []
+            self._system_prompt = None
+        self._stateless_memory = None
+
+        if system_prompt:
+            logger.debug(f"Initialized agent '{self.name}' with model '{str(model)}' "
+                         f"using custom system prompt (persona disabled).")
+            logger.debug("Prompt: " + self.prompt())
+        elif persona:
+            logger.debug(f"Initialized agent '{self.name}' with model '{str(model)}' "
+                         f"using prompt from '{config['prompts']['persona_agent']}'.")
+            logger.debug("Prompt: " + self.prompt())
+        else:
+            logger.debug("Initialized agent with no persona and no system prompt.")
 
         self.add_orchestrators(orchestrators)
         self.add_inspectors(inspectors)
         self.reset()
 
+    @property
+    def memory(self) -> List[BaseMessage]:
+        """
+        The conversation memory as a list of messages.
+        """
+        return self._memory if self._stateless_memory is None else self._stateless_memory
+
     @property
     def _hooked_responses(self):
         """
@@ -248,23 +282,31 @@ class Agent:
             pass
         return None
 
-    def __call__(self, utterance: str = "", return_events: bool = False) -> str:
+    def __call__(self, utterance: Union[str, List[BaseMessage]] = "", return_events: bool = False) -> str:
         """
         Processes an input utterance and generates a response.
 
-        :param utterance: The input utterance from the other agent or user.
-        :type utterance: str
+        :param utterance: The input utterance from the other agent or, in case of stateless operation,
+                          the full context as a list of Langchain messages.
+        :type utterance: Union[str, List[BaseMessage]]
         :param return_events: If True, returns a list of events instead of just the response string.
         :type return_events: bool
         :return: The agent's response or events, or None if finished.
         :rtype: Union[str, List[Event], None]
         """
-        if self._finished:
-            return None
+        # If stateless
+        if isinstance(utterance, list):
+            if len(self._memory) > 0:
+                utterance.insert(0, self._memory[0])  # Always keep the original system prompt / persona
+            self._stateless_memory = utterance
+        else:
+            self._stateless_memory = None
+            if self._finished:
+                return None
 
-        if utterance:
-            utterance = self._preprocessing_fn(utterance) if self._preprocessing_fn else utterance
-            self.memory.append(HumanMessage(content=utterance))
+            if utterance:
+                utterance = self._preprocessing_fn(utterance) if self._preprocessing_fn else utterance
+                self.memory.append(HumanMessage(content=utterance))
 
         if return_events:
             events = []
@@ -386,6 +428,7 @@ class Agent:
                                 timestamp=int(time())))
 
         if hasattr(response, "tool_calls") and response.tool_calls and self._tools:
+            messages.append(response)
             messages_n = len(messages)
             for tool_call in response.tool_calls:
                 events.append(Event(agent=self.get_name(),
@@ -436,7 +479,8 @@ class Agent:
             self.add_orchestrators(other)
         return self
 
-    def _add_activation_hooks(self, key_to_layer_name, steering_function=None, steering_interval=(0, -1)):
+    def _add_activation_hooks(self, key_to_layer_name, steering_function=None,
+                              steering_interval=(0, -1), inspector=None):
         """
         Registers ActivationHooks for each layer in the given mapping.
         Skips already registered layers.
@@ -447,25 +491,42 @@ class Agent:
         :type steering_function: Optional[Callable]
         :param steering_interval: (min_token, max_token). Skip first min_token; stop after max_token (-1 = no limit).
         :type steering_interval: Tuple[int, int]
+        :param inspector: Inspector instance that owns these hooks (for accessing top_k).
+        :type inspector: Optional[Inspector]
         """
         # Get the model (assume HuggingFace pipeline)
         if self.base_model is self.llm:
             raise RuntimeError("Base model not found or not a HuggingFace pipeline.")
 
-        # Always re-initialize cache and hooks
-        self._hook_acts = []
+        # Initialize hooks list if it doesn't exist (it is the case when 2 or more inpsectors are involved)
+        if not hasattr(self, '_hook_acts'):
+            self._hook_acts = []
 
-        # Register new hooks
+        # Register new hooks, but skip if a hook for this layer already exists
         for cache_key, layer_name in key_to_layer_name.items():
-            hook = ActivationHook(
-                cache_key=cache_key,
-                layer_key=layer_name,
-                agent=self,
-                response_hook=self._hook_response_data,
-                steering_function=steering_function,  # pass the function here,
-                steering_interval=steering_interval
-            )
-            self._hook_acts.append(hook)
+            # Check if a hook for this layer already exists
+            existing_hook = None
+            for existing in self._hook_acts:
+                if existing.layer_key == layer_name:
+                    existing_hook = existing
+                    break
+
+            if existing_hook is None:
+                # Use layer name as cache key to ensure uniqueness across inspectors
+                # This prevents collisions when multiple inspectors use the same cache key
+                # but for different layers
+                unique_cache_key = layer_name
+                # Create new hook for this layer
+                hook = ActivationHook(
+                    cache_key=unique_cache_key,
+                    layer_key=layer_name,
+                    agent=self,
+                    response_hook=self._hook_response_data,
+                    steering_function=steering_function,
+                    steering_interval=steering_interval,
+                    inspector=inspector
+                )
+                self._hook_acts.append(hook)
 
     def _clear_hooks(self):
         """
@@ -474,16 +535,47 @@ class Agent:
         for hook in getattr(self, '_hook_acts', []):
             hook.remove()
         self._hook_acts = []
+        # Clear logits hooks if they exist
+        for hook in getattr(self, '_hook_logits', []):
+            hook.remove()
+        self._hook_logits = []
+        self._hook_response_logit.clear()
+        self._hook_response_logit.update(defaultdict(list))
         if self._hook_response_data is not None:
             self._hook_response_data.reset()
         self._set_hook_response_data()
 
-    def _set_hook_response_data(self):
+    def _set_hook_response_data(self, inspector=None):
         """
         Ensures a ResponseHook is registered (idempotent).
+
+        :param inspector: Inspector instance that owns this hook (for accessing top_k).
+        :type inspector: Optional[Inspector]
         """
         if self._hook_response_data is None:
-            self._hook_response_data = ResponseHook(agent=self)
+            self._hook_response_data = ResponseHook(agent=self, inspector=inspector)
+
+    def serve(self,
+              host: str = "0.0.0.0",
+              port: int = 1333,
+              stateless: bool = True,
+              log_level: str = "info"):
+        """
+        Starts a REST API server to interact with the agent.
+
+        :param host: Host address to bind the server to.
+        :type host: str
+        :param port: Port number to listen on.
+        :type port: int
+        :param stateless: If True, the server does not maintain conversation state (as such the full context
+                          must be provided with each request).
+        :type stateless: bool
+        :param log_level: Logging level for the server.
+        :type log_level: str
+        """
+        from .server import Server
+
+        return Server.serve(agents=self, host=host, port=port, stateless=stateless, log_level=log_level)
 
     def response_lookahead(self, message: str = None):
         """
@@ -635,7 +727,7 @@ class Agent:
         data["model"] = self.model_info
         if self._first_utterances:
             data["first_utterances"] = self._first_utterances
-        data["persona"] = self.persona.json()
+        data["persona"] = self.persona.json() if self.persona else {}
         if self._orchestrators:
             data["persona"]["orchestrators"] = [orc.json() for orc in self._orchestrators]
         return json.dumps(data, indent=indent) if string else data
@@ -649,10 +741,15 @@ class Agent:
         :param context: Optional context override.
         :param example_dialogs: Optional replacement example dialogs for prompt regeneration.
         """
+
         # Remove history
-        self.memory[:] = self.memory[:1]
+        if self._system_prompt is None:
+            self.memory.clear()
+        else:
+            self.memory[:] = self.memory[:1]
+
         # Update system prompt if needed
-        if self.memory and (context or example_dialogs):
+        if self.persona and self.memory and (context or example_dialogs):
             system_prompt = self._system_prompt_template.render(
                 persona=self.persona.prompt(),
                 context=context or self._context,
@@ -769,8 +866,9 @@ class Agent:
             model=self.model_info,
             seed=seed,
             personas={
-                self.get_name(): self.persona.json(),
-                agent.get_name(default="Other"): agent.persona.json()},
+                self.get_name(): self.persona.json() if self.persona else {},
+                agent.get_name(default="Other"): agent.persona.json() if agent.persona else {}
+            },
             context=context.json() if context and isinstance(context, Context) else context,
             scenario=scenario,
             notes=notes,
