@@ -105,8 +105,8 @@ def _kl_divergence(p1, p2, resolution=100, bw_method=1e-1):
 
 class LinguisticFeatureScore(BaseDialogScore):
     """
-    Compute linguistic features for a dialogue: mean turn length, hesitation rate,
-    Gunning Fog index, and Flesch Reading Ease score.
+    Compute one or multiple simple linguistic features for a dialogue: mean turn length,
+    hesitation rate, Gunning Fog index, and Flesch Reading Ease score.
 
     Example:
 
@@ -114,31 +114,53 @@ class LinguisticFeatureScore(BaseDialogScore):
 
             from sdialog.evaluation import LinguisticFeatureScore
 
-            scorer_all_feat = LinguisticFeatureScore()
-            scorer_hesit = LinguisticFeatureScore(feature="hesitation-rate")
+            # All features as a dictionary
+            scorer_all = LinguisticFeatureScore()
+            # Single feature
+            scorer_hes = LinguisticFeatureScore(feature="hesitation-rate")
+            # Subset of features
+            scorer_subset = LinguisticFeatureScore(feature=["mean-turn-length", "gunning-fog"])
 
-            print(scorer_all_feat(dialog))  # dict of values for all linguistic features
-            print(scorer_hesit(dialog))     # single float value for hesitation rate
+            print(scorer_all(dialog))      # dict with all feature values
+            print(scorer_hes(dialog))      # single float (hesitation rate)
+            print(scorer_subset(dialog))   # dict with the two requested features
 
-    :param feature: Specific feature to compute or None to compute all (default).
-                    Features are one of:
+    :param feature: List of feature names to compute. If ``None`` (default) compute all.
+                    If the resulting set has size 1, ``__call__`` / ``score`` returns a single float; otherwise a dict.
+                    Available features:
 
-                      - ``"mean-turn-length"`` (average words per turn),
-                      - ``"hesitation-rate"`` (percentage of hesitation tokens over total words),
-                      - ``"gunning-fog"`` (Gunning Fog readability index),
-                      - ``"flesch-reading-ease"`` (Flesch Reading Ease score);
-    :type feature: Optional[Literal["mean-turn-length","hesitation-rate","gunning-fog","flesch-reading-ease"]]
-    :param name: Internal score name.
+                      - ``"mean-turn-length"``: average number of words per dialogue turn.
+                      - ``"hesitation-rate"``: percentage of hesitation tokens over total words (%).
+                      - ``"gunning-fog"``: Gunning Fog readability index.
+                      - ``"flesch-reading-ease"``: Flesch Reading Ease score.
+
+    :type feature: Optional[List[Literal["mean-turn-length", "hesitation-rate", "gunning-fog", "flesch-reading-ease"]]]
+    :param name: Internal score name (defaults to ``"linguistic_features"`` or
+                 the single feature name if only one provided).
     :type name: str
     :param speaker: If set, only turns by this speaker (case-insensitive) are considered.
     :type speaker: Optional[str]
     """
     def __init__(self,
-                 feature: Literal["mean-turn-length", "hesitation-rate", "gunning-fog", "flesch-reading-ease"] = None,
-                 name="linguistic_features",
+                 feature: Optional[List[Literal["mean-turn-length", "hesitation-rate",
+                                                "gunning-fog", "flesch-reading-ease"]]] = None,
+                 name: str = None,
                  speaker: Optional[str] = None):
         """Initialize scorer."""
-        super().__init__(name=name or feature or "")
+        if isinstance(feature, str):
+            feature = [feature]
+
+        # Check all features valid
+        valid_features = {"mean-turn-length", "hesitation-rate", "gunning-fog", "flesch-reading-ease"}
+        if feature and not all(f in valid_features for f in feature):
+            raise ValueError(f"Invalid feature(s): {feature}. Must be one or more of: {valid_features}")
+
+        # If a single feature is requested, allow name override with that feature for clearer downstream tables
+        if name is None:
+            if feature and isinstance(feature, list) and len(feature) == 1:
+                name = feature[0]
+
+        super().__init__(name=name or "")
         self.feature = feature
         self.speaker = speaker
 
@@ -168,7 +190,7 @@ class LinguisticFeatureScore(BaseDialogScore):
         :return: Gunning Fog index value.
         :rtype: float
         """
-        sentences = re.split(r'[.!?]+', text)
+        sentences = re.split(r'[.!?\n]+', text)
         sentences = [s.strip() for s in sentences if s.strip()]
         if not sentences:
             return 0
@@ -189,7 +211,7 @@ class LinguisticFeatureScore(BaseDialogScore):
         :return: Reading ease score.
         :rtype: float
         """
-        sentences = [s.strip() for s in re.split(r'[.!?]+', text) if s.strip()]
+        sentences = [s.strip() for s in re.split(r'[.!?\n]+', text) if s.strip()]
         if not sentences:
             return 0
         words = re.findall(r'\b[a-zA-Z]+\b', text)
@@ -200,30 +222,30 @@ class LinguisticFeatureScore(BaseDialogScore):
         avg_syllables_per_word = total_syllables / len(words)
         return 206.835 - (1.015 * avg_sentence_length) - (84.6 * avg_syllables_per_word)
 
-    def score(self, dialog: Dialog) -> float:
-        """
-        Compute one or multiple linguistic features for the dialogue.
+    def score(self, dialog: Dialog) -> Union[float, dict]:
+        """Compute one or multiple linguistic features for the dialogue.
 
         :param dialog: Dialogue instance to evaluate.
         :type dialog: Dialog
-        :return: Feature value (float) if single feature requested; otherwise dict mapping feature -> value.
+        :return: If exactly one feature is requested (either because ``feature`` was a singleton list
+                 or because a single feature key is computed), returns a single ``float``.
+                 Otherwise returns a ``dict`` mapping feature-name (canonical snake/dash variants preserved)
+                 to numeric value.
         :rtype: Union[float, dict]
         """
         if self.speaker:
             dialog = dialog.filter(speaker=self.speaker)
 
-        utts = [" ".join(re.findall(r"[\w-]+", turn.text)) for turn in dialog]
-
         results = {}
-        all_text = " ".join(utts)
-        turn_lengths = [len(utt.split()) for utt in utts]
-        if not self.feature or self.feature == "mean-turn-length":
+        all_text = str(dialog)
+        turn_lengths = [len(turn) for turn in dialog]
+        if not self.feature or "mean-turn-length" in self.feature:
             results["mean-turn-length"] = np.mean(turn_lengths)
-        if not self.feature or self.feature == "hesitation-rate":
+        if not self.feature or "hesitation-rate" in self.feature:
             results["hesitation_rate"] = (self.count_hesitations(all_text) / max(1, sum(turn_lengths)) * 100)
-        if not self.feature or self.feature == "gunning-fog":
+        if not self.feature or "gunning-fog" in self.feature:
             results["gunning_fog"] = self.calculate_gunning_fog(all_text)
-        if not self.feature or self.feature == "flesch-reading-ease":
+        if not self.feature or "flesch-reading-ease" in self.feature:
             results["flesch_reading_ease"] = self.calculate_flesch_reading_ease(all_text)
 
         return results if len(results) > 1 else list(results.values())[0]
@@ -484,6 +506,12 @@ class LLMJudgeYesNo(BaseDialogScore, BaseLLMJudge):
         output = BaseLLMJudge.__call__(self, prompt)
         output = self.output_format.model_validate(output)
 
+        if isinstance(output.positive, list) and not output.positive:
+            if len(dialogs) > 1:
+                output.positive = [False] * len(dialogs)
+            else:
+                output.positive = False
+
         return output
 
     @CacheDialogScore.cache
@@ -1024,13 +1052,13 @@ class ReferenceCentroidEmbeddingEvaluator(BaseDatasetEmbeddingEvaluator):
         for i, label in enumerate(unique_labels):
             idx = all_labels == label
             plt.scatter(tsne_embs[idx, 0], tsne_embs[idx, 1],
-                        label=label if label != "reference" else None,
+                        label=label,
                         alpha=0.15 if label == "reference" else 0.7,
                         color="black" if label == "reference" else colors[i % len(colors)])
         for label in ["reference"] + list(dialog_embs.keys()):
             idx = all_labels == f"centroid-{label}"
             plt.scatter(tsne_embs[idx, 0], tsne_embs[idx, 1],
-                        label="Reference centroid" if label == "reference" else None,
+                        label="reference centroid" if label == "reference" else None,
                         linewidths=3 if label == "reference" else 2,
                         alpha=1,  # if label == "reference" else 0.7,
                         color="black" if label == "reference" else colors[unique_labels.index(label) % len(colors)],
@@ -1674,6 +1702,7 @@ class StatsEvaluator(BaseDatasetScoreEvaluator):
         plot.title(f"Boxplot of {title}")
         plot.boxplot(list(dialog_scores.values()),
                      labels=list(dialog_scores.keys()))
+        plt.xticks(rotation=45, ha='right')
         plot.xlabel("datasets")
         plot.ylabel(name or self.dialog_score.name)
 
@@ -1686,12 +1715,15 @@ class StatsEvaluator(BaseDatasetScoreEvaluator):
         :return: Dict of statistics, or a single value if a target statistic was specified.
         :rtype: Union[dict, float]
         """
+        if len(dialog_scores) == 0:
+            logger.warning(f"No valid scores to compute statistics for {self.name}. Returning zeros.")
+
         stats = {
-            "mean": np.mean(dialog_scores),
-            "std": np.std(dialog_scores),
-            "min": np.min(dialog_scores),
-            "max": np.max(dialog_scores),
-            "median": np.median(dialog_scores)
+            "mean": np.mean(dialog_scores) if len(dialog_scores) > 0 else 0.0,
+            "std": np.std(dialog_scores) if len(dialog_scores) > 0 else 0.0,
+            "min": np.min(dialog_scores) if len(dialog_scores) > 0 else 0.0,
+            "max": np.max(dialog_scores) if len(dialog_scores) > 0 else 0.0,
+            "median": np.median(dialog_scores) if len(dialog_scores) > 0 else 0.0
         }
         return stats[self.stat] if self.stat in stats else stats
 
@@ -1788,6 +1820,7 @@ class FrequencyEvaluator(BaseDatasetScoreEvaluator):
         for bar in bars:
             height = bar.get_height()
             plot.text(bar.get_x() + bar.get_width() / 2, height, f"{height:.1f}%", ha='center', va='bottom')
+        plot.xticks(rotation=45, ha='right')
         plot.ylabel(f"Percentage of {metric or self.dialog_score.name} (%)")
         plot.xlabel("datasets")
         plot.title(f"Percentage of {metric or self.dialog_score.name} per dataset")
@@ -1836,15 +1869,15 @@ class DatasetComparator:
 
             comparator.plot()  # plot results for each evaluator that support it
 
-    :param evaluators: List of evaluator instances.
-    :type evaluators: List[BaseDatasetEvaluator]
-    :raises ValueError: If list empty.
-    :raises TypeError: If any element not a BaseDatasetEvaluator.
+    :param evaluators: Single evaluator instance or list of evaluator instances.
+    :type evaluators: Union[BaseDatasetEvaluator, List[BaseDatasetEvaluator]]
     """
-    def __init__(self, evaluators: List[BaseDatasetEvaluator]):
+    def __init__(self, evaluators: Union[BaseDatasetEvaluator, List[BaseDatasetEvaluator]]):
         """Initialize dataset comparator."""
         if not evaluators:
             raise ValueError("No evaluators provided for comparison.")
+        if not isinstance(evaluators, list):
+            evaluators = [evaluators]
         for evaluator in evaluators:
             if not isinstance(evaluator, BaseDatasetEvaluator):
                 raise TypeError(f"Evaluator {evaluator} is not an instance of `BaseDatasetEvaluator`")
@@ -1888,13 +1921,13 @@ class DatasetComparator:
                 dataset_name += 1
             results[dataset_name] = {}
             for evaluator in self.evaluators:
-                evaluator_name = evaluator.name
                 score = evaluator(dataset, dataset_name=dataset_name)
                 if isinstance(score, dict):
                     for metric, value in score.items():
-                        results[dataset_name][f"{evaluator_name}-{metric}"] = value
+                        metric = f"{evaluator.name}-{metric}" if evaluator.name else metric
+                        results[dataset_name][metric] = value
                 else:
-                    results[dataset_name][evaluator_name] = score
+                    results[dataset_name][evaluator.name] = score
 
         if output == "dict" or output is dict:
             return results
